@@ -1,29 +1,38 @@
 from __future__ import print_function
+from __future__ import unicode_literals
 
+import codecs
+import io
 import json
 import os
 import subprocess
 import sys
 from builtins import input
 from collections import defaultdict
+from copy import deepcopy
 
-from ..plugins.core import initialize
-from ..plugins.high_entropy_strings import HighEntropyStringsPlugin
-from ..plugins.keyword import determine_file_type
-from ..plugins.keyword import KeywordDetector
-from .baseline import format_baseline_for_output
+try:
+    from functools import lru_cache
+except ImportError:  # pragma: no cover
+    from functools32 import lru_cache
+
+from ..plugins.common import initialize
+from ..plugins.common.util import get_mapping_from_secret_type_to_class_name
+from ..util import get_git_remotes
+from ..util import get_git_sha
 from .baseline import merge_results
 from .bidirectional_iterator import BidirectionalIterator
+from .code_snippet import CodeSnippetHighlighter
 from .color import AnsiColor
 from .color import colorize
-from .potential_secret import PotentialSecret
+from .common import write_baseline_to_file
 
 
 class SecretNotFoundOnSpecifiedLineError(Exception):
     def __init__(self, line):
         super(SecretNotFoundOnSpecifiedLineError, self).__init__(
-            "ERROR: Secret not found on line {}!\n".format(line) +
-            "Try recreating your baseline to fix this issue.",
+            'ERROR: Secret not found on line {}!\n'.format(line)
+            + 'Try recreating your baseline to fix this issue.',
         )
 
 
@@ -31,6 +40,37 @@ class RedundantComparisonError(Exception):
     pass
 
 
+AUDIT_RESULT_TO_STRING = {
+    True: 'true-positives',
+    False: 'false-positives',
+    None: 'unknowns',
+}
+
+EMPTY_PLUGIN_AUDIT_RESULT = {
+    'results': {
+        'true-positives': defaultdict(list),
+        'false-positives': defaultdict(list),
+        'unknowns': defaultdict(list),
+    },
+    'config': {},
+}
+EMPTY_STATS_RESULT = {
+    'signal': 0,
+    'true-positives': {
+        'count': 0,
+        'files': defaultdict(int),
+    },
+    'false-positives': {
+        'count': 0,
+        'files': defaultdict(int),
+    },
+    'unknowns': {
+        'count': 0,
+        'files': defaultdict(int),
+    },
+}
+
+
 def audit_baseline(baseline_filename):
     original_baseline = _get_baseline_from_file(baseline_filename)
     if not original_baseline:
@@ -62,7 +102,10 @@ def audit_baseline(baseline_filename):
             )
             decision = _get_user_decision(can_step_back=secret_iterator.can_step_back())
         except SecretNotFoundOnSpecifiedLineError:
-            decision = _get_user_decision(prompt_secret_decision=False)
+            decision = _get_user_decision(
+                prompt_secret_decision=False,
+                can_step_back=secret_iterator.can_step_back(),
+            )
 
         if decision == 'q':
             print('Quitting...')
@@ -87,7 +130,10 @@ def audit_baseline(baseline_filename):
         original_baseline['results'],
         dict(results),
     )
-    _save_baseline_to_file(baseline_filename, original_baseline)
+    write_baseline_to_file(
+        filename=baseline_filename,
+        data=original_baseline,
+    )
 
 
 def compare_baselines(old_baseline_filename, new_baseline_filename):
@@ -172,6 +218,124 @@ def compare_baselines(old_baseline_filename, new_baseline_filename):
             secret_iterator.step_back_on_next_iteration()
 
 
+def determine_audit_results(baseline, baseline_path):
+    """
+    Given a baseline which has been audited, returns
+    a dictionary describing the results of each plugin in the following form:
+    {
+        "plugins": {
+            "plugin_name1": {
+                "results": {
+                    "true-positives": [
+                        list of {
+                            filename: {
+                                'line': '...',
+                                'plaintext':'...',
+                            }
+                        } for secrets with `is_secret: true` caught by this plugin],
+                    "false-positives": [
+                        list of {
+                            filename: {
+                                'line': '...',
+                                'plaintext':'...',
+                            }
+                        } for secrets with `is_secret: false` caught by this plugin],
+                    "unknowns": [
+                        list of {
+                            filename: {
+                                'line': '...',
+                                'plaintext':'...',
+                            }
+                        } for secrets with no `is_secret` entry caught by this plugin]
+                },
+                "config": {configuration used for the plugin}
+            },
+            ...
+        },
+        "repo_info": {
+            "remote": "remote url",
+            "sha": "sha of repo checkout"
+        },
+    }
+    """
+    all_secrets = _secret_generator(baseline)
+
+    audit_results = {
+        'plugins': defaultdict(lambda: deepcopy(EMPTY_PLUGIN_AUDIT_RESULT)),
+        'stats': deepcopy(EMPTY_STATS_RESULT),
+    }
+
+    secret_type_to_plugin_name = get_mapping_from_secret_type_to_class_name()
+
+    total = 0
+    for filename, secret in all_secrets:
+        file_contents = _open_file_with_cache(filename)
+
+        secret_info = {}
+        secret_info['line'] = _get_file_line(filename, secret['line_number'])
+        try:
+            secret_info['plaintext'] = get_raw_secret_value(
+                secret=secret,
+                plugin_settings=baseline['plugins_used'],
+                file_handle=io.StringIO(file_contents),
+                filename=filename,
+            )
+        except SecretNotFoundOnSpecifiedLineError:
+            secret_info['plaintext'] = None
+
+        plugin_name = secret_type_to_plugin_name[secret['type']]
+        audit_result = AUDIT_RESULT_TO_STRING[secret.get('is_secret')]
+        audit_results['plugins'][plugin_name]['results'][audit_result][filename].append(secret_info)
+
+        audit_results['stats'][audit_result]['count'] += 1
+        audit_results['stats'][audit_result]['files'][filename] += 1
+        total += 1
+    audit_results['stats']['signal'] = str(
+        (
+            float(audit_results['stats']['true-positives']['count'])
+            /
+            total
+        ) * 100,
+    )[:4] + '%'
+
+    for plugin_config in baseline['plugins_used']:
+        plugin_name = plugin_config['name']
+        if plugin_name not in audit_results['plugins']:
+            continue
+
+        audit_results['plugins'][plugin_name]['config'].update(plugin_config)
+
+    git_repo_path = os.path.dirname(os.path.abspath(baseline_path))
+    git_sha = get_git_sha(git_repo_path)
+    git_remotes = get_git_remotes(git_repo_path)
+
+    if git_sha and git_remotes:
+        audit_results['repo_info'] = {
+            'remote': git_remotes[0],
+            'sha': git_sha,
+        }
+
+    return audit_results
+
+
+def print_audit_results(baseline_filename):
+    baseline = _get_baseline_from_file(baseline_filename)
+    if not baseline:
+        print('Failed to retrieve baseline from {filename}'.format(filename=baseline_filename))
+        return
+
+    print(
+        json.dumps(
+            determine_audit_results(
+                baseline,
+                baseline_filename,
+            ),
+            indent=2,
+            sort_keys=True,
+        ),
+    )
+
+
 def _get_baseline_from_file(filename):  # pragma: no cover
     try:
         with open(filename) as f:
@@ -233,7 +397,7 @@ def _get_secrets_to_compare(old_baseline, new_baseline):
         if not new_filename:
             secrets_to_compare += list(
                 map(
-                    lambda x: (old_filename, x, True,),
+                    lambda x: (old_filename, x, True),
                     old_baseline['results'][old_filename],
                 ),
             )
@@ -241,7 +405,7 @@ def _get_secrets_to_compare(old_baseline, new_baseline):
         elif not old_filename:
             secrets_to_compare += list(
                 map(
-                    lambda x: (new_filename, x, False,),
+                    lambda x: (new_filename, x, False),
                     new_baseline['results'][new_filename],
                 ),
             )
@@ -258,11 +422,11 @@ def _get_secrets_to_compare(old_baseline, new_baseline):
 
             if old_secret:
                 secrets_to_compare.append(
-                    (old_filename, old_secret, True,),
+                    (old_filename, old_secret, True),
                 )
             else:
                 secrets_to_compare.append(
-                    (new_filename, new_secret, False,),
+                    (new_filename, new_secret, False),
                 )
 
     return secrets_to_compare
@@ -288,22 +452,22 @@ def _comparison_generator(old_list, new_list, compare_fn):
 
         status = compare_fn(old_value, new_value)
         if status == 0:
-            yield (old_value, new_value,)
+            yield (old_value, new_value)
             old_index += 1
             new_index += 1
         elif status == -1:
-            yield (old_value, None,)
+            yield (old_value, None)
             old_index += 1
         else:
-            yield (None, new_value,)
+            yield (None, new_value)
             new_index += 1
 
     # Catch leftovers. Only one of these while statements should run.
     while old_index < len(old_list):
-        yield (old_list[old_index], None,)
+        yield (old_list[old_index], None)
         old_index += 1
     while new_index < len(new_list):
-        yield (None, new_list[new_index],)
+        yield (None, new_list[new_index])
         new_index += 1
 
 
@@ -346,16 +510,18 @@ def _print_context(  # pragma: no cover
 
     :raises: SecretNotFoundOnSpecifiedLineError
     """
-    print('{} {} {} {}\n{} {}\n{} {}'.format(
-        colorize('Secret:     ', AnsiColor.BOLD),
-        colorize(str(count), AnsiColor.PURPLE),
-        colorize('of', AnsiColor.BOLD),
-        colorize(str(total), AnsiColor.PURPLE),
-        colorize('Filename:   ', AnsiColor.BOLD),
-        colorize(filename, AnsiColor.PURPLE),
-        colorize('Secret Type:', AnsiColor.BOLD),
-        colorize(secret['type'], AnsiColor.PURPLE),
-    ))
+    print(
+        '{} {} {} {}\n{} {}\n{} {}'.format(
+            colorize('Secret:     ', AnsiColor.BOLD),
+            colorize(str(count), AnsiColor.PURPLE),
+            colorize('of', AnsiColor.BOLD),
+            colorize(str(total), AnsiColor.PURPLE),
+            colorize('Filename:   ', AnsiColor.BOLD),
+            colorize(filename, AnsiColor.PURPLE),
+            colorize('Secret Type:', AnsiColor.BOLD),
+            colorize(secret['type'], AnsiColor.PURPLE),
+        ),
+    )
     if additional_header_lines:
         print(additional_header_lines)
 
@@ -420,9 +586,30 @@ def _handle_user_decision(decision, secret):
         del secret['is_secret']
 
 
-def _save_baseline_to_file(filename, data):  # pragma: no cover
-    with open(filename, 'w') as f:
-        f.write(format_baseline_for_output(data))
+@lru_cache(maxsize=1)
+def _open_file_with_cache(filename):
+    """
+    Reads the input file and returns the result as a string.
+
+    This caches opened files to ensure that the audit functionality
+    doesn't unnecessarily re-open the same file.
+    """
+    try:
+        with codecs.open(filename, encoding='utf-8') as f:
+            return f.read()
+    except (OSError, IOError):
+        return None
+
+
+def _get_file_line(filename, line_number):
+    """
+    Attempts to read a given line from the input file.
+    """
+    file_content = _open_file_with_cache(filename)
+    if not file_content:
+        return None
+
+    return file_content.splitlines()[line_number - 1]
 
 
 def _get_secret_with_context(
@@ -454,146 +641,73 @@ def _get_secret_with_context(
 
     :raises: SecretNotFoundOnSpecifiedLineError
     """
-    secret_lineno = secret['line_number']
-    start_line = 1 if secret_lineno <= lines_of_context \
-        else secret_lineno - lines_of_context
-    end_line = secret_lineno + lines_of_context
-
-    output = subprocess.check_output([
-        'sed',
-        '-n', '{},{}p'.format(start_line, end_line),
-        filename,
-    ]).decode('utf-8').splitlines()
-
-    trailing_lines_of_context = lines_of_context
-    if len(output) < end_line - start_line + 1:
-        # This handles the case of a short file.
-        num_lines_in_file = int(subprocess.check_output([
-            # https://stackoverflow.com/a/38870057
-            # - 'wc -l' cannot be used here because if the last char
-            # of the file isn't \n, then the last line isn't counted
-            'grep',
-            '-c',
-            '',
-            filename,
-        ]).decode('utf-8').split()[0])
-
-        trailing_lines_of_context = lines_of_context - \
-            (end_line - num_lines_in_file)
-
-    # -1, because that's where the secret actually is (without it,
-    # it would just be the start of the context block).
-    # NOTE: index_of_secret_in_output should *always* be negative.
-    index_of_secret_in_output = -trailing_lines_of_context - 1
 
     try:
-        output[index_of_secret_in_output] = _highlight_secret(
-            output[index_of_secret_in_output],
-            secret_lineno,
-            secret,
-            filename,
-            plugin_settings,
+        file_content = _open_file_with_cache(filename)
+        if not file_content:
+            raise SecretNotFoundOnSpecifiedLineError(secret['line_number'])
+
+        file_lines = file_content.splitlines()
+
+        snippet = CodeSnippetHighlighter().get_code_snippet(
+            file_lines,
+            secret['line_number'],
+            lines_of_context=lines_of_context,
         )
+
+        raw_secret_value = get_raw_secret_value(
+            secret,
+            plugin_settings,
+            io.StringIO(file_content),
+            filename,
+        )
+
+        try:
+            snippet.highlight_line(raw_secret_value)
+        except ValueError:
+            raise SecretNotFoundOnSpecifiedLineError(secret['line_number'])
     except SecretNotFoundOnSpecifiedLineError:
         if not force:
             raise
 
-        output[index_of_secret_in_output] = '{}'.format(
-            colorize(
-                output[index_of_secret_in_output],
-                AnsiColor.BOLD,
-            ),
+        snippet.target_line = colorize(
+            snippet.target_line,
+            AnsiColor.BOLD,
         )
 
-    # Adding line numbers
-    return '\n'.join(
-        map(
-            lambda x: '{}:{}'.format(
-                colorize(
-                    str(int(x[0]) + start_line),
-                    AnsiColor.LIGHT_GREEN,
-                ),
-                x[1],
-            ),
-            enumerate(output),
-        ),
-    )
+    return snippet.add_line_numbers()
 
 
-def _highlight_secret(
-    secret_line,
-    secret_lineno,
+def get_raw_secret_value(
     secret,
-    filename,
     plugin_settings,
+    file_handle,
+    filename,
 ):
     """
-    :type secret_line: str
-    :param secret_line: the line on which the secret is found
-
-    :type secret_lineno: int
-    :param secret_lineno: secret_line's line number in the source file
-
     :type secret: dict
     :param secret: see caller's docstring
 
-    :type filename: str
-    :param filename: this is needed, because PotentialSecret uses this
-        as a means of comparing whether two secrets are equal.
-
     :type plugin_settings: list
     :param plugin_settings: see caller's docstring
 
-    :rtype: str
-    :returns: secret_line, but with the actual secret highlighted.
+    :type file_handle: file object
+    :param file_handle: Open handle to file where the secret is
+
+    :type filename: str
+    :param filename: this is needed, because PotentialSecret uses this
+        as a means of comparing whether two secrets are equal.
     """
     plugin = initialize.from_secret_type(
         secret['type'],
         plugin_settings,
     )
 
-    for raw_secret in _raw_secret_generator(
-        plugin,
-        secret_line,
-        filetype=determine_file_type(filename),
-    ):
-        secret_obj = PotentialSecret(
-            plugin.secret_type,
-            filename,
-            secret=raw_secret,
-        )
+    plugin_secrets = plugin.analyze(file_handle, filename)
 
-        # There could be more than two secrets on the same line.
-        # We only want to highlight the right one.
-        if secret_obj.secret_hash == secret['hashed_secret']:
-            break
-    else:
-        raise SecretNotFoundOnSpecifiedLineError(secret_lineno)
+    # Return value of matching secret
+    for plugin_secret in plugin_secrets:
+        if plugin_secret.secret_hash == secret['hashed_secret']:
+            return plugin_secret.secret_value
 
-    index_of_secret = secret_line.lower().index(raw_secret.lower())
-    end_of_secret = index_of_secret + len(raw_secret)
-    return '{}{}{}'.format(
-        secret_line[:index_of_secret],
-        colorize(
-            # copy the secret out of the line because .lower() from secret
-            # generator may be different from the original value:
-            secret_line[index_of_secret:end_of_secret],
-            AnsiColor.RED_BACKGROUND,
-        ),
-        secret_line[index_of_secret + len(raw_secret):],
-    )
-
-
-def _raw_secret_generator(plugin, secret_line, filetype):
-    """Generates raw secrets by re-scanning the line, with the specified plugin"""
-    if isinstance(plugin, KeywordDetector):
-        for raw_secret in plugin.secret_generator(secret_line, filetype=filetype):
-            yield raw_secret
-    else:
-        for raw_secret in plugin.secret_generator(secret_line):
-            yield raw_secret
-
-    if issubclass(plugin.__class__, HighEntropyStringsPlugin):
-        with plugin.non_quoted_string_regex(strict=False):
-            for raw_secret in plugin.secret_generator(secret_line):
-                yield raw_secret
+    raise SecretNotFoundOnSpecifiedLineError(secret['line_number'])
