@@ -5,50 +5,81 @@ import os
 import re
 import subprocess
 
+from detect_secrets import util
+from detect_secrets.core.log import get_logger
 from detect_secrets.core.secrets_collection import SecretsCollection
 
 
+log = get_logger(format_string='%(message)s')
+
+
 def initialize(
+    path,
     plugins,
-    exclude_regex=None,
-    path='.',
-    scan_all_files=False,
+    exclude_files_regex=None,
+    exclude_lines_regex=None,
+    word_list_file=None,
+    word_list_hash=None,
+    should_scan_all_files=False,
 ):
     """Scans the entire codebase for secrets, and returns a
     SecretsCollection object.
 
+    :type path: list
+
     :type plugins: tuple of detect_secrets.plugins.base.BasePlugin
     :param plugins: rules to initialize the SecretsCollection with.
 
-    :type exclude_regex: str|None
-    :type path: str
-    :type scan_all_files: bool
+    :type exclude_files_regex: str|None
+    :type exclude_lines_regex: str|None
+
+    :type word_list_file: str|None
+    :param word_list_file: optional word list file for ignoring certain words.
+
+    :type word_list_hash: str|None
+    :param word_list_hash: optional iterated sha1 hash of the words in the word list.
+
+    :type should_scan_all_files: bool
 
     :rtype: SecretsCollection
     """
-    output = SecretsCollection(plugins, exclude_regex)
+    output = SecretsCollection(
+        plugins,
+        exclude_files=exclude_files_regex,
+        exclude_lines=exclude_lines_regex,
+        word_list_file=word_list_file,
+        word_list_hash=word_list_hash,
+    )
 
-    if os.path.isfile(path):
-        # This option allows for much easier adhoc usage.
-        files_to_scan = [path]
-    elif scan_all_files:
-        files_to_scan = _get_files_recursively(path)
-    else:
-        files_to_scan = _get_git_tracked_files(path)
+    files_to_scan = []
+    for element in path:
+        if os.path.isdir(element):
+            if should_scan_all_files:
+                files_to_scan.extend(
+                    _get_files_recursively(element),
+                )
+            else:
+                files_to_scan.extend(
+                    _get_git_tracked_files(element),
+                )
+        elif os.path.isfile(element):
+            files_to_scan.append(element)
+        else:
+            log.error('detect-secrets: %s: No such file or directory', element)
 
     if not files_to_scan:
         return output
 
-    if exclude_regex:
-        regex = re.compile(exclude_regex, re.IGNORECASE)
+    if exclude_files_regex:
+        exclude_files_regex = re.compile(exclude_files_regex, re.IGNORECASE)
         files_to_scan = filter(
             lambda file: (
-                not regex.search(file)
+                not exclude_files_regex.search(file)
             ),
             files_to_scan,
         )
 
-    for file in files_to_scan:
+    for file in sorted(files_to_scan):
         output.scan_file(file)
 
     return output
@@ -66,13 +97,13 @@ def get_secrets_not_in_baseline(results, baseline):
     :rtype: SecretsCollection
     :returns: SecretsCollection of new results (filtering out baseline)
     """
-    regex = None
-    if baseline.exclude_regex:
-        regex = re.compile(baseline.exclude_regex, re.IGNORECASE)
+    exclude_files_regex = None
+    if baseline.exclude_files:
+        exclude_files_regex = re.compile(baseline.exclude_files, re.IGNORECASE)
 
     new_secrets = SecretsCollection()
     for filename in results.data:
-        if regex and regex.search(filename):
+        if exclude_files_regex and exclude_files_regex.search(filename):
             continue
 
         if filename not in baseline.data:
@@ -161,7 +192,7 @@ def merge_baseline(old_baseline, new_baseline):
     """Updates baseline to be compatible with the latest version of
     detect-secrets.
 
-    Currently, this only exists to transfer whitelisted secrets across
+    Currently, this only exists to transfer allowlisted secrets across
     to the new baseline, and will only work with baselines created
     after v0.9.
 
@@ -172,6 +203,8 @@ def merge_baseline(old_baseline, new_baseline):
 
     :type new_baseline: dict
     :param new_baseline: most recent scan
+
+    :rtype: dict
     """
     new_baseline['results'] = merge_results(
         old_baseline['results'],
@@ -196,7 +229,7 @@ def merge_results(old_results, new_results):
         if filename not in new_results:
             continue
 
-        old_secrets_mapping = dict()
+        old_secrets_mapping = {}
         for old_secret in old_secrets:
             old_secrets_mapping[old_secret['hashed_secret']] = old_secret
 
@@ -223,7 +256,7 @@ def format_baseline_for_output(baseline):
     for filename, secret_list in baseline['results'].items():
         baseline['results'][filename] = sorted(
             secret_list,
-            key=lambda x: (x['line_number'], x['hashed_secret'],),
+            key=lambda x: (x['line_number'], x['hashed_secret']),
         )
 
     return json.dumps(
@@ -231,7 +264,7 @@ def format_baseline_for_output(baseline):
         indent=2,
         sort_keys=True,
         separators=(',', ': '),
-    ) + '\n'
+    )
 
 
 def _get_git_tracked_files(rootdir='.'):
@@ -248,20 +281,24 @@ def _get_git_tracked_files(rootdir='.'):
     :rtype: set|None
     :returns: filepaths to files which git currently tracks (locally)
     """
+    output = []
     try:
         with open(os.devnull, 'w') as fnull:
             git_files = subprocess.check_output(
                 [
                     'git',
+                    '-C', rootdir,
                     'ls-files',
-                    rootdir,
                 ],
                 stderr=fnull,
             )
-
-        return set(git_files.decode('utf-8').split())
+        for filename in git_files.decode('utf-8').split():
+            relative_path = util.get_relative_path_if_in_cwd(rootdir, filename)
+            if relative_path:
+                output.append(relative_path)
     except subprocess.CalledProcessError:
-        return None
+        pass
+    return output
 
 
 def _get_files_recursively(rootdir):
@@ -269,8 +306,9 @@ def _get_files_recursively(rootdir):
     This function allows us to do so.
     """
     output = []
-    for root, dirs, files in os.walk(rootdir):
+    for root, _, files in os.walk(rootdir):
         for filename in files:
-            output.append(os.path.join(root, filename))
-
+            relative_path = util.get_relative_path_if_in_cwd(root, filename)
+            if relative_path:
+                output.append(relative_path)
     return output
