@@ -1,7 +1,7 @@
 class BidirectionalIterator(object):
     def __init__(self, collection):
         self.collection = collection
-        self.index = -1  # starts on -1, as index is increased _before_ getting result
+        self.index = -1  # Starts on -1, as index is increased _before_ getting result
         self.step_back_once = False
 
     def __next__(self):
@@ -21,7 +21,7 @@ class BidirectionalIterator(object):
 
         return result
 
-    def next(self):
+    def next(self):  # pragma: no cover
         return self.__next__()
 
     def step_back_on_next_iteration(self):
@@ -30,5 +30,5 @@ class BidirectionalIterator(object):
     def can_step_back(self):
         return self.index > 0
 
-    def __iter__(self):
+    def __iter__(self):  # pragma: no cover
         return self
