@@ -8,28 +8,45 @@ from time import gmtime
 from time import strftime
 
 from detect_secrets import VERSION
+from detect_secrets.core.constants import IGNORED_FILE_EXTENSIONS
 from detect_secrets.core.log import log
 from detect_secrets.core.potential_secret import PotentialSecret
-from detect_secrets.plugins.core import initialize
+from detect_secrets.plugins.common import initialize
+from detect_secrets.util import build_automaton
 
 
 class SecretsCollection(object):
 
-    def __init__(self, plugins=(), exclude_regex=''):
+    def __init__(
+        self,
+        plugins=(),
+        exclude_files=None,
+        exclude_lines=None,
+        word_list_file=None,
+        word_list_hash=None,
+    ):
         """
         :type plugins: tuple of detect_secrets.plugins.base.BasePlugin
         :param plugins: rules to determine whether a string is a secret
 
-        :type exclude_regex: str
-        :param exclude_regex: for optional regex for ignored paths.
+        :type exclude_files: str|None
+        :param exclude_files: optional regex for ignored paths.
 
-        :type version: str
-        :param version: version of detect-secrets that SecretsCollection
-            is valid at.
+        :type exclude_lines: str|None
+        :param exclude_lines: optional regex for ignored lines.
+
+        :type word_list_file: str|None
+        :param word_list_file: optional word list file for ignoring certain words.
+
+        :type word_list_hash: str|None
+        :param word_list_hash: optional iterated sha1 hash of the words in the word list.
         """
         self.data = {}
         self.plugins = plugins
-        self.exclude_regex = exclude_regex
+        self.exclude_files = exclude_files
+        self.exclude_lines = exclude_lines
+        self.word_list_file = word_list_file
+        self.word_list_hash = word_list_hash
         self.version = VERSION
 
     @classmethod
@@ -43,13 +60,13 @@ class SecretsCollection(object):
         :raises: IOError
         """
         try:
-            return cls._load_baseline_from_dict(json.loads(string))
+            return cls.load_baseline_from_dict(json.loads(string))
         except (IOError, ValueError):
             log.error('Incorrectly formatted baseline!')
             raise
 
     @classmethod
-    def _load_baseline_from_dict(cls, data):
+    def load_baseline_from_dict(cls, data):
         """Initializes a SecretsCollection object from dictionary.
 
         :type data: dict
@@ -59,22 +76,53 @@ class SecretsCollection(object):
         :raises: IOError
         """
         result = SecretsCollection()
-        if not all(key in data for key in (
-            'exclude_regex',
-            'plugins_used',
-            'results',
-        )):
+
+        if not all(
+            key in data for key in (
+                'plugins_used',
+                'results',
+            )
+        ):
             raise IOError
 
-        result.exclude_regex = data['exclude_regex']
+        # In v0.12.0 `exclude_regex` got replaced by `exclude`
+        if not any(
+            key in data for key in (
+                'exclude',
+                'exclude_regex',
+            )
+        ):
+            raise IOError
+
+        if 'exclude_regex' in data:
+            result.exclude_files = data['exclude_regex']
+        else:
+            result.exclude_files = data['exclude']['files']
+            result.exclude_lines = data['exclude']['lines']
+
+        # In v0.12.7 the `--word-list` option got added
+        automaton = None
+        if 'word_list' in data:
+            result.word_list_file = data['word_list']['file']
+            result.word_list_hash = data['word_list']['hash']
+
+            if result.word_list_file:
+                # Always ignore the given `data['word_list']['hash']`
+                # The difference will show whenever the word list changes
+                automaton, result.word_list_hash = build_automaton(result.word_list_file)
 
         plugins = []
         for plugin in data['plugins_used']:
             plugin_classname = plugin.pop('name')
-            plugins.append(initialize.from_plugin_classname(
-                plugin_classname,
-                **plugin
-            ))
+            plugins.append(
+                initialize.from_plugin_classname(
+                    plugin_classname,
+                    exclude_lines_regex=result.exclude_lines,
+                    automaton=automaton,
+                    should_verify_secrets=False,
+                    **plugin
+                ),
+            )
         result.plugins = tuple(plugins)
 
         for filename in data['results']:
@@ -141,13 +189,13 @@ class SecretsCollection(object):
             log.error(alert)
             raise
 
-        if self.exclude_regex:
-            regex = re.compile(self.exclude_regex, re.IGNORECASE)
+        if self.exclude_files:
+            regex = re.compile(self.exclude_files, re.IGNORECASE)
 
         for patch_file in patch_set:
             filename = patch_file.path
-            # If the file matches the exclude_regex, we skip it
-            if self.exclude_regex and regex.search(filename):
+            # If the file matches the exclude_files, we skip it
+            if self.exclude_files and regex.search(filename):
                 continue
 
             if filename == baseline_filename:
@@ -179,14 +227,15 @@ class SecretsCollection(object):
 
         if os.path.islink(filename):
             return False
-
+        if os.path.splitext(filename)[1] in IGNORED_FILE_EXTENSIONS:
+            return False
         try:
             with codecs.open(filename, encoding='utf-8') as f:
                 self._extract_secrets_from_file(f, filename_key)
 
             return True
         except IOError:
-            log.warning("Unable to open file: %s", filename)
+            log.warning('Unable to open file: %s', filename)
             return False
 
     def get_secret(self, filename, secret, type_=None):
@@ -233,15 +282,24 @@ class SecretsCollection(object):
         for key in results:
             results[key] = sorted(results[key], key=lambda x: x['line_number'])
 
-        plugins_used = list(map(
-            lambda x: x.__dict__,
-            self.plugins,
-        ))
+        plugins_used = list(
+            map(
+                lambda x: x.__dict__,
+                self.plugins,
+            ),
+        )
         plugins_used = sorted(plugins_used, key=lambda x: x['name'])
 
         return {
-            'generated_at': strftime("%Y-%m-%dT%H:%M:%SZ", gmtime()),
-            'exclude_regex': self.exclude_regex,
+            'generated_at': strftime('%Y-%m-%dT%H:%M:%SZ', gmtime()),
+            'exclude': {
+                'files': self.exclude_files,
+                'lines': self.exclude_lines,
+            },
+            'word_list': {
+                'file': self.word_list_file,
+                'hash': self.word_list_hash,
+            },
             'plugins_used': plugins_used,
             'results': results,
             'version': self.version,
@@ -276,14 +334,14 @@ class SecretsCollection(object):
         :type filename: string
         """
         try:
-            log.info("Checking file: %s", filename)
+            log.info('Checking file: %s', filename)
 
             for results, plugin in self._results_accumulator(filename):
                 results.update(plugin.analyze(f, filename))
                 f.seek(0)
 
         except UnicodeDecodeError:
-            log.warning("%s failed to load.", filename)
+            log.warning('%s failed to load.', filename)
 
     def _extract_secrets_from_patch(self, f, plugin, filename):
         """Extract secrets from a given patch file object.
@@ -300,7 +358,7 @@ class SecretsCollection(object):
             for line in chunk.target_lines():
                 if line.is_added:
                     output.update(
-                        plugin.analyze_string(
+                        plugin.analyze_line(
                             line.value,
                             line.target_line_no,
                             filename,
@@ -317,7 +375,7 @@ class SecretsCollection(object):
 
             for secret_hash in self.data[filename]:
                 tmp = self.data[filename][secret_hash].json()
-                del tmp['filename']  # not necessary
+                del tmp['filename']  # Because filename will map to the secrets
 
                 output[filename].append(tmp)
 
