@@ -1,24 +1,28 @@
 from __future__ import absolute_import
+from __future__ import unicode_literals
 
+import os
 import subprocess
 import sys
 import textwrap
 
 from detect_secrets import VERSION
-from detect_secrets.core.baseline import format_baseline_for_output
 from detect_secrets.core.baseline import get_secrets_not_in_baseline
 from detect_secrets.core.baseline import trim_baseline_of_removed_secrets
+from detect_secrets.core.common import write_baseline_to_file
 from detect_secrets.core.log import get_logger
 from detect_secrets.core.secrets_collection import SecretsCollection
 from detect_secrets.core.usage import ParserBuilder
-from detect_secrets.plugins.core import initialize
+from detect_secrets.plugins.common import initialize
+from detect_secrets.util import build_automaton
 
 
 log = get_logger(format_string='%(message)s')
 
 
 def parse_args(argv):
-    return ParserBuilder().add_pre_commit_arguments()\
+    return ParserBuilder()\
+        .add_pre_commit_arguments()\
         .parse_args(argv)
 
 
@@ -35,7 +39,27 @@ def main(argv=None):
         # Error logs handled within logic.
         return 1
 
-    plugins = initialize.from_parser_builder(args.plugins)
+    automaton = None
+    word_list_hash = None
+    if args.word_list_file:
+        automaton, word_list_hash = build_automaton(args.word_list_file)
+
+    plugins = initialize.from_parser_builder(
+        args.plugins,
+        exclude_lines_regex=args.exclude_lines,
+        automaton=automaton,
+        should_verify_secrets=not args.no_verify,
+    )
+
+    # Merge plugins from baseline
+    if baseline_collection:
+        plugins = initialize.merge_plugins_from_baseline(
+            baseline_collection.plugins,
+            args,
+            automaton,
+        )
+        baseline_collection.plugins = plugins
+
     results = find_secrets_in_files(args, plugins)
     if baseline_collection:
         original_results = results
@@ -59,14 +83,13 @@ def main(argv=None):
     )
 
     if VERSION != baseline_collection.version:
-        baseline_collection.plugins = plugins
         baseline_collection.version = VERSION
         baseline_modified = True
 
     if baseline_modified:
-        _write_to_baseline_file(
-            args.baseline[0],
-            baseline_collection.format_for_baseline_output(),
+        write_baseline_to_file(
+            filename=args.baseline[0],
+            data=baseline_collection.format_for_baseline_output(),
         )
 
         log.error(
@@ -74,17 +97,11 @@ def main(argv=None):
             'Probably to keep line numbers of secrets up-to-date.\n'
             'Please `git add {}`, thank you.\n\n'.format(args.baseline[0]),
         )
-        return 1
+        return 3
 
     return 0
 
 
-def _write_to_baseline_file(filename, payload):  # pragma: no cover
-    """Breaking this function up for mockability."""
-    with open(filename, 'w') as f:
-        f.write(format_baseline_for_output(payload))
-
-
 def get_baseline(baseline_filename):
     """
     :raises: IOError
@@ -133,7 +150,7 @@ def raise_exception_if_baseline_file_is_unstaged(filename):
                 '--name-only',
             ],
         ).split()
-    except subprocess.CalledProcessError:
+    except subprocess.CalledProcessError:  # pragma: no cover
         # Since we don't pipe stderr, we get free logging through git.
         raise ValueError
 
@@ -175,7 +192,7 @@ def pretty_print_diagnostics(secrets):
 def _print_warning_header():
     message = (
         'Potential secrets about to be committed to git repo! Please rectify '
-        'or explicitly ignore with an inline `pragma: whitelist secret` comment.'
+        'or explicitly ignore with an inline `pragma: allowlist secret` comment.'
     )
 
     log.error(textwrap.fill(message))
@@ -189,9 +206,15 @@ def _print_secrets_found(secrets):
 
 
 def _print_mitigation_suggestions():
+    security_team = os.environ.get(
+        'DETECT_SECRETS_SECURITY_TEAM',
+        'in #security',
+    )
     suggestions = [
-        'For information about putting your secrets in a safer place, please ask in #security',
-        'Mark false positives with an inline `pragma: whitelist secret` comment',
+        'For information about putting your secrets in a safer place, ' +
+        'please ask ' + security_team,
+        'Mark false positives with an inline ' +
+        '`pragma: allowlist secret` comment',
         'Commit with `--no-verify` if this is a one-time false positive',
     ]
 
